"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const tmp = tslib_1.__importStar(require("tmp"));
const minimist_1 = tslib_1.__importDefault(require("minimist"));
const path = tslib_1.__importStar(require("path"));
const glob_1 = tslib_1.__importDefault(require("glob"));
const fs = tslib_1.__importStar(require("fs"));
const cp_file_1 = tslib_1.__importDefault(require("cp-file"));
const childProcess = tslib_1.__importStar(require("child_process"));
const rimraf = tslib_1.__importStar(require("rimraf"));
const npm_version_cli_1 = require("npm-version-cli");
const semver = tslib_1.__importStar(require("semver"));
const clean_scripts_1 = require("clean-scripts");
const packageJson = tslib_1.__importStar(require("../package.json"));
function showToolVersion() {
    console.log(`Version: ${packageJson.version}`);
}
function showHelp() {
    console.log(`Version ${packageJson.version}
Syntax:   clean-release [options]
Examples: clean-release
          clean-release --config clean-release.config.js
          clean-release --config clean-release.config.ts
          clean-release --only-changed-packages
          clean-release --effected-packages foo --effected-packages bar
Options:
 -h, --help                                         Print this message.
 -v, --version                                      Print the version
 --config                                           Config file
 --only-changed-packages                            Only changed packages will bump.
 --effected-packages                                The packages will be considered as effected packages.
`);
}
function globAsync(pattern, ignore) {
    return new Promise((resolve, reject) => {
        (0, glob_1.default)(pattern, { ignore }, (error, matches) => {
            if (error) {
                reject(error);
            }
            else {
                resolve(matches);
            }
        });
    });
}
function statAsync(file) {
    return new Promise((resolve) => {
        fs.stat(file, (error, stats) => {
            if (error) {
                resolve(undefined);
            }
            else {
                resolve(stats);
            }
        });
    });
}
const subProcesses = [];
function exec(command, options) {
    return new Promise((resolve, reject) => {
        console.log(`${command}...`);
        const subProcess = childProcess.exec(command, options || {}, (error, stdout) => {
            if (error) {
                reject(error);
            }
            else {
                resolve(stdout);
            }
        });
        if (subProcess.stdout) {
            subProcess.stdout.pipe(process.stdout);
        }
        if (subProcess.stderr) {
            subProcess.stderr.pipe(process.stderr);
        }
        subProcesses.push(subProcess);
    });
}
async function executeCommandLine() {
    const argv = (0, minimist_1.default)(process.argv.slice(2), { '--': true });
    const showVersion = argv.v || argv.version;
    if (showVersion) {
        showToolVersion();
        return;
    }
    if (argv.h || argv.help) {
        showHelp();
        return;
    }
    let configFilePath;
    if (argv.config) {
        configFilePath = path.resolve(process.cwd(), argv.config);
    }
    else {
        configFilePath = path.resolve(process.cwd(), 'clean-release.config.ts');
        const stats = await statAsync(configFilePath);
        if (!stats || !stats.isFile()) {
            configFilePath = path.resolve(process.cwd(), 'clean-release.config.js');
        }
    }
    if (configFilePath.endsWith('.ts')) {
        try {
            require('ts-node/register/transpile-only');
        }
        catch {
            require('@esbuild-kit/cjs-loader');
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-var-requires
    let configData = require(configFilePath);
    if (configData.default) {
        configData = configData.default;
    }
    const packageJsonPath = path.resolve(process.cwd(), 'package.json');
    // eslint-disable-next-line @typescript-eslint/no-var-requires
    const packageJsonData = require(packageJsonPath);
    if (configData.changesGitStaged) {
        const status = (await exec(`git status -s`, configData.execOptions)).trim();
        if (status && status.split('\n').some(s => !s.startsWith('A ') && !s.startsWith('M '))) {
            throw new Error('There are changes not staged for commit.');
        }
    }
    let effectedWorkspacePaths;
    if (configData.askVersion) {
        const effectedPackages = [];
        if (typeof argv["effected-packages"] === 'string') {
            effectedPackages.push(argv["effected-packages"]);
        }
        else if (Array.isArray(argv["effected-packages"])) {
            effectedPackages.push(...argv["effected-packages"]);
        }
        const options = {
            onlyChangedPackages: !!argv['only-changed-packages'] || configData.onlyChangedPackages,
            effectedPackages,
        };
        const { version, effectedWorkspaces } = await (0, npm_version_cli_1.askVersion)(options);
        packageJsonData.version = version;
        if (effectedWorkspaces) {
            effectedWorkspacePaths = effectedWorkspaces.map((w) => w.map((e) => e.path));
        }
    }
    const result = tmp.dirSync();
    console.log(`Tmp Dir: ${result.name}`);
    try {
        if (configData.releaseRepository) {
            if (configData.releaseBranchName) {
                await exec(`git clone -b ${configData.releaseBranchName} ${configData.releaseRepository} "${result.name}" --depth=1`, configData.execOptions);
            }
            else {
                await exec(`git clone ${configData.releaseRepository} "${result.name}" --depth=1`, configData.execOptions);
            }
            const fileOrDirectories = fs.readdirSync(result.name);
            for (const fileOrDirectory of fileOrDirectories) {
                const fullpath = path.resolve(result.name, fileOrDirectory);
                if (!fs.statSync(fullpath).isDirectory() || fileOrDirectory !== '.git') {
                    rimraf.sync(fullpath);
                }
            }
        }
        if (!configData.include || configData.include.length === 0) {
            throw new Error('Expect at least one pattern.');
        }
        const uniqFiles = await globAsync(configData.include.length === 1 ? configData.include[0] : `{${configData.include.join(',')}}`, configData.exclude);
        for (const file of uniqFiles) {
            if (!fs.existsSync(file)) {
                throw new Error(`Error: file: "${file}" not exists.`);
            }
            if (fs.statSync(file).isFile()) {
                let relativePath = path.relative('.', path.dirname(file));
                if (configData.base) {
                    if (typeof configData.base === 'string') {
                        if (relativePath.startsWith(configData.base)) {
                            relativePath = path.relative(configData.base, relativePath);
                        }
                    }
                    else if (Array.isArray(configData.base)) {
                        for (const base of configData.base) {
                            if (relativePath.startsWith(base)) {
                                relativePath = path.relative(base, relativePath);
                            }
                        }
                    }
                    else {
                        relativePath = configData.base(relativePath);
                    }
                }
                const directoryPath = path.resolve(result.name, relativePath);
                fs.mkdirSync(directoryPath, { recursive: true });
                await (0, cp_file_1.default)(file, path.resolve(directoryPath, path.basename(file)));
                console.log(`Copied: ${file} To:  ${relativePath}`);
            }
        }
        if (configData.releaseRepository) {
            await exec(`cd ${result.name} && git add -A --force && git commit -m "${packageJsonData.version}" && git tag -a v${packageJsonData.version} -m 'v${packageJsonData.version}' && git push && git push origin v${packageJsonData.version}`, configData.execOptions);
        }
        if (configData.postScript) {
            const versionData = semver.parse(packageJsonData.version);
            const tag = versionData && versionData.prerelease.length > 0 ? versionData.prerelease[0] : undefined;
            if (Array.isArray(configData.postScript)) {
                const totalTime = [];
                for (const postScript of configData.postScript) {
                    if (!postScript) {
                        continue;
                    }
                    if (typeof postScript === 'string') {
                        await exec(fillScript(postScript, result.name, packageJsonData.version), configData.execOptions);
                    }
                    else {
                        const script = await postScript({
                            dir: result.name,
                            version: packageJsonData.version,
                            effectedWorkspacePaths,
                            tag
                        });
                        if (script) {
                            const times = await (0, clean_scripts_1.executeScriptAsync)(script, undefined, undefined, undefined, configData.execOptions);
                            totalTime.push(...times);
                        }
                    }
                }
                (0, clean_scripts_1.logTimes)(totalTime);
            }
            else if (configData.postScript) {
                if (typeof configData.postScript === 'string') {
                    await exec(fillScript(configData.postScript, result.name, packageJsonData.version), configData.execOptions);
                }
                else {
                    const script = await configData.postScript({
                        dir: result.name,
                        version: packageJsonData.version,
                        effectedWorkspacePaths,
                        tag
                    });
                    if (script) {
                        const times = await (0, clean_scripts_1.executeScriptAsync)(script, undefined, undefined, undefined, configData.execOptions);
                        (0, clean_scripts_1.logTimes)(times);
                    }
                }
            }
        }
        if (!configData.notClean) {
            rimraf.sync(result.name);
        }
    }
    catch (error) {
        if (!configData.notClean) {
            rimraf.sync(result.name);
        }
        throw error;
    }
}
function fillScript(script, dir, version) {
    return script.split('[dir]').join(dir).split('[version]').join(version);
}
function cleanup() {
    if (process.platform === 'darwin' || process.platform === 'linux') {
        const stdout = childProcess.execSync('ps -l').toString();
        const ps = stdout.split('\n')
            .map(s => s.split(' ').filter(s => s))
            .filter((s, i) => i > 0 && s.length >= 2)
            .map(s => ({ pid: +s[1], ppid: +s[2] }));
        const result = [];
        collectPids(process.pid, ps, result);
        for (const pid of result) {
            childProcess.execSync(`kill -9 ${pid}`);
        }
    }
    for (const subProcess of subProcesses) {
        subProcess.kill('SIGINT');
    }
}
executeCommandLine().then(() => {
    cleanup();
    process.exit();
}, (error) => {
    if (error instanceof Error) {
        console.log(error.message);
    }
    else {
        console.log(error);
    }
    cleanup();
    process.exit(1);
});
function collectPids(pid, ps, result) {
    const children = ps.filter(p => p.ppid === pid);
    for (const child of children) {
        result.push(child.pid);
        collectPids(child.pid, ps, result);
    }
}
process.on('SIGINT', () => {
    cleanup();
    process.exit();
});
process.on('SIGTERM', () => {
    cleanup();
    process.exit();
});
